/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Client side Tunnel.
 *
 * $Id: ClientTunnel.hh,v 1.16 2007-11-16 10:39:41 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _CLIENT_TUNNEL_HH_
#define _CLIENT_TUNNEL_HH_

#include "Tunnel.hh"

namespace HTTPTunnel {

class ClientTunneler;

class ClientTunnel : public Tunnel {
public:
	// Construction/destruction
	ClientTunnel(TunnelConfig* tcfg, ClientTunneler* tunneler);
	virtual ~ClientTunnel();


	/***********/
	/* Methods */
	/***********/

	// Process some newly extracted tunnel data from one of our connections
	virtual void processTunnelMsg(HTTPMsg* msg, HTTPConnection* httpConn);

	// Poll the far end of the tunnel
	void pollTunnel(void);

	// Set whether the tunnel user is authenticated
	void setUserAuthenticated(bool auth);


protected:

	/*************/
	/* Constants */
	/*************/

   // Time between poll timeout events (milli-seconds)
   static const unsigned int		MIN_POLL_TIMEOUT				= 200;
   static const unsigned int		MAX_POLL_TIMEOUT				= 2500;

	// Time between queued data tunnel msg sends (milli-seconds)
	static const unsigned int		EARLY_MSG_SEND_TIME			= 25;


	/***********/
	/* Methods */
	/***********/

	// Record that we've sent a new tunnel msg
	virtual void sentNewTunnelMsg(HTTPMsgParms* msgParms);

	// Determine whether a multi-ack implies we should resend a msg
	virtual bool checkForMultiAckResendRequest(SentMsgParms* sentMsgParms);

	// Have we got a stable HTTP connection to other end?
	virtual bool httpStable(void) const;

	// Can we send a new tunnel msg?
	virtual bool clearToSendNewMsg(Connection* conn);

	// Check whether any connections need to be blocked/unblocked
	virtual void checkBlockedConnections(void);

	// Advance our internal state machine
	virtual void advanceState(const HTTPMsgParms* parms);

	// Get maximum allowable payload size
	virtual unsigned int getMaxPayloadSize(bool useHeader);

	// Something has been sent down the tunnel
	virtual void tunnelAnyMsgSent(void);

	// Scale poll timeout
	void scalePollTimeout(void);


	/********/
	/* Data */
	/********/

	// Time between poll timeout events
   unsigned int						m_pollTimeout;


private:

	/********/
	/* Data */
	/********/

	// Flag to indicate if we have a clear & stable HTTP connection
	bool							m_httpConnStable;

	// Time next poll is scheduled for
	Time							m_nextPollTime;
	bool							m_polling;

};


}	// namespace

#endif

