/* (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate parameters for an HTTP message.
 *
 * $Id: HTTPMsgParms.hh,v 1.23 2007-10-19 15:32:55 rich Exp $
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _HTTP_MSG_PARMS_HH_
#define _HTTP_MSG_PARMS_HH_

#include <string>
#include "MsgParms.hh"
#include "Tunnel.hh"
#include "User.hh"
#include "Quotas.hh"
#include "Enums.hh"

namespace HTTPTunnel {

class Payload;
class ProxySettings;
class ServerAuthSettings;

class HTTPMsgParms : public MsgParms {
public:
	// Construction/destruction
	HTTPMsgParms();
	HTTPMsgParms(std::string tid, Payload* payload, bool retry, ProxySettings* proxyCfg,
		ServerAuthSettings* serverAuthCfg, User& user, Tunnel::TunnelState state,
		Tunnel::TunnelTestState testState, std::string ver, std::string ack,
		std::string tunDest, bool useHeader, Tunnel* tunnel);
	HTTPMsgParms(const HTTPMsgParms& that);
	virtual ~HTTPMsgParms();

	// Assignment
	HTTPMsgParms& operator=(const HTTPMsgParms& that);

	// Clone
	virtual MsgParms* clone(void) const;

	// Accessors
	Tunnel::TunnelState getState(void) const;
	Tunnel::TunnelTestState getTestState(void) const;
	std::string getVer(void) const;
	std::string getAck(void) const;
	std::string getTunDest(void) const;
	bool getUseHeaderFields(void) const;
	ProxySettings* getProxySettings(void);
	ServerAuthSettings* getServerAuthSettings(void);
	const User& getUser(void) const;
	const Quotas& getQuotas(void) const;
	unsigned short getRespCode(void) const;
	std::string getRespPhrase(void) const;
	std::string getURI(void) const;
	std::string getCliVer(void) const;
	bool getUserAuthenticated(void) const;
	bool getConnClose(void) const;
	bool getConnKeepAlive(void) const;
	bool getProxyConnClose(void) const;
	bool getProxyConnKeepAlive(void) const;

	// Mutators
	void setState(Tunnel::TunnelState state);
	void setTestState(Tunnel::TunnelTestState testState);
	void setVer(std::string ver);
	void setAck(std::string ack);
	void setTunDest(std::string dest);
	void setUseHeaderFields(bool uh);
	void setRespCodePhrase(unsigned short code, std::string phrase);
	void setURI(std::string uri);
	void setProxySettings(ProxySettings* settings);
	void setServerAuthSettings(ServerAuthSettings* settings);
	void setUser(const User& user);
	void setQuotas(const Quotas& quotas);
	void setCliVer(std::string cliVer);
	void setConnOption(std::string opt);
	void setProxyConnOption(std::string opt);


protected:

	/********/
	/* Data */
	/********/

	// Connection states for tunnel
	Tunnel::TunnelState			m_state;
	Tunnel::TunnelTestState		m_testState;

	// HTTP version
	std::string						m_ver;

	// Message ack
	std::string						m_ack;

	// Tunnel destination address
	std::string						m_tunDest;

	// Using header fields for tunneling?
	bool								m_useHeaderFields;

	// Tunnel proxy & serverauth settings (tunnel maintains ownership)
	ProxySettings*					m_proxyCfg;
	ServerAuthSettings*			m_serverAuthCfg;

	// User msg is for
	User								m_user;
	Quotas							m_quotas;

	// Code & phrase for response msgs
	unsigned short					m_respCode;
	std::string						m_respPhrase;

	// Requested URI (either for sending or repsonding to)
	std::string						m_uri;

	// Client version identifier
	std::string						m_cliVer;

	// Tunnel msg is for
	Tunnel*							m_tunnel;

	// Connection options
	bool								m_connClose;
	bool								m_connKeepAlive;
	bool								m_proxyConnClose;
	bool								m_proxyConnKeepAlive;

};


}	// namespace

#endif

