/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * HTTP Tunneler instance. Top level library interface.
 *
 * $Id: HTTPTunneler.hh,v 1.2 2007-11-30 15:14:42 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _HTTP_TUNNELER_HH_
#define _HTTP_TUNNELER_HH_

#include "Mutex.hh"
#include "Exceptions.hh"

namespace HTTPTunnel {

class HTTPTunneler {
public:

	/**********************************/
	/* HTTPTunneler instance handling */
	/**********************************/

	// Get/Release singleton instance
	static HTTPTunneler* Get(void) throw(TunnelError);
	static void Release(void) throw(TunnelError);


	/*******************************/
	/* Control the HTTP tunnelling */
	/*******************************/

	// Start/Stop the background HTTP tunnelling
	void Start(void) throw(TunnelError);
	void Stop(void) throw(TunnelError);


	/*********************************/
	/* Configure the HTTP tunnelling */
	/*********************************/

	// Configure from a file
	void ConfigureFromFile(const char* fileName) throw(TunnelError);

	// Regenerate the named config file with defaults
	void RegenerateCfgFile(const char* fileName) throw(TunnelError);

#if 0
	// Configure directly
	void ConfigureGlobal(void) throw(TunnelError);
	unsigned int ConfigureClientTunnel(void) throw(TunnelError);
	unsigned int ConfigureServerTunnel(void) throw(TunnelError);

	// Reconfigure tunnels directly
	void ReconfigureClientTunnel(void) throw(TunnelError);
	void ReconfigureServerTunnel(void) throw(TunnelError);

	// Remove configured tunnels directly
	void RemoveTunnel(void) throw(TunnelError);
#endif


private:

	/***********/
	/* Methods */
	/***********/

	// Construction/destruction
	HTTPTunneler() throw(TunnelError);
	~HTTPTunneler() throw(TunnelError);

	// Perform one-off initialisation/shutdown
	void initialise(void) throw(TunnelError);
	void shutdown(void) throw(TunnelError);

	// PID file management
	void createPIDFile(void) throw(TunnelError);
	void removePIDFile(void);

	// Daemonise
	void daemonise(void) throw(TunnelError);

	#ifdef DEBUG_LOG_DATA
	// Open data debugging logs
	void openDebugLogs(void);
	#endif


	/********/
	/* Data */
	/********/

	// Mutex
	Mutex								m_mtx;

	// Name of any PID file
	std::string						m_pidFileName;

	// Flag to indicate whether we have started running
	bool								m_running;

	// Singleton instance
	static HTTPTunneler*			c_tunneler;
	static Mutex					c_mtx;

};


}	// namespace

#endif

