/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * A convenient place to store all global configuration for
 * one side of a tunneler.
 *
 * $Id: SideConfig.hh,v 1.5 2007-11-30 15:14:42 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _SIDE_CONFIG_HH_
#define _SIDE_CONFIG_HH_

#include <string>
#include <fstream>
#include "Enums.hh"
#include "Exceptions.hh"

namespace HTTPTunnel {

class Authenticator;

class SideConfig {
public:
	// Construction/destruction
	SideConfig(Enums::Protocol prot, bool ssl, std::string lhost, unsigned int lport);
	virtual ~SideConfig();


	/*************/
	/* Accessors */
	/*************/

	// Get protocol to use for server
	Enums::Protocol getProtocol(void) const;
	bool getSSL(void) const;

	// Get listen address/port
	std::string getListenIP(void) const;
	unsigned int getListenPort(void) const;
	std::string getListenAddr(void) const;

	// Client or server config?
	virtual bool isClient(void) const;
	virtual bool isServer(void) const;

	// Get authenticator
	virtual Authenticator* getAuthenticator(void);


	/************/
	/* Mutators */
	/************/

	// Set listen address/port
	void setListenIP(std::string ip);
	void setListenPort(unsigned int lport);


	/*************/
	/* Utilities */
	/*************/

	// Do any required late-construction of data
	virtual void doLateConstruction(void) throw(TunnelError) = 0;

	// Serialize to a file
	virtual void serialize(std::ofstream& file) = 0;


protected:

	/********/
	/* Data */
	/********/

	// Protocol to use for server comms
	Enums::Protocol				m_protocol;
	bool								m_ssl;

	// The local listening address/port
	std::string						m_listenIP;
	unsigned int					m_listenPort;

	// Authenticator
	Authenticator*					m_authenticator;

};


}	// namespace

#endif

