/* (C) 2003 Kevin Manning.
 * (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Implement the functions of a thread.
 *
 * $Id: Thread.cc,v 1.5 2007-11-24 13:58:26 rich Exp $
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <iostream>
#include "Thread.hh"
#include "ThreadManager.hh"

namespace HTTPTunnel {

// Construction/destruction
Thread::Thread(ThreadManager* manager, Callback& cbf) throw(ThreadError)
: m_manager(manager), m_callback(cbf.clone()), m_thread(0)
{
	#ifdef DEBUG_THREADS
	std::cerr << "Thread::Thread()" << std::endl;
	#endif

	// Create thread
	if(pthread_create(&m_thread, 0, Thread::threadMain, this) != 0)
		throw ThreadError("pthread_create() failed");
}

Thread::~Thread()
{
	#ifdef DEBUG_THREADS
	std::cerr << "Thread::~Thread()" << std::endl;
	#endif

	// We own the cloned callback
	delete m_callback;

	// Let the manager know we're done
	m_manager->finished(this);

	// We don't bother with things like return values...
	pthread_detach(pthread_self());
}


// Start the thread running
void Thread::start(void)
{
	#ifdef DEBUG_THREADS
	std::cerr << "Thread::start()" << std::endl;
	#endif

	m_callback->operator()();
}


/*********************/
/* Private interface */
/*********************/


// Start off point for all threads
void* Thread::threadMain(void* args)
{
	#ifdef DEBUG_THREADS
	std::cerr << "Entering Thread::threadMain()" << std::endl;
	#endif

	// Get Thread* and set thread specific data
	Thread* thread = static_cast<Thread*>(args);

	// Dont throw any exceptions ourselves
	try {
		// Let the manager know
		thread->m_manager->started(thread);

		// Let the thread call it's proper work function
		thread->start();
	}
	catch(ThreadError& e) {
		#ifdef DEBUG_THREADS
		std::cerr << "ThreadError caught: " << e.what() << std::endl;
		#endif
	}

	// Tidy up
	delete thread;

	#ifdef DEBUG_THREADS
	std::cerr << "Leaving Thread::threadMain()" << std::endl;
	#endif

	return 0;
}


}	// namespace

